// -------------------- Global Variables

// Helper Objects
var fs = new ActiveXObject("Scripting.FileSystemObject");
var st = DOpus.Create.StringTools();

// Columns' Definition
var config = {
	columns: [{
	name:"ToAddress",
	displayName:"To Address",
	re: /^To:\s(.*)$/,
	type: "string",
	defwidth: 18
	}, {
	name:"FromAddress",
	displayName:"From Address",
	re: /^From:\s(.*)$/,
	type: "string",
	defwidth: 18
	}, {
	name:"Subject",
	displayName:"Subject",
	re: /^Subject:\s(.*)$/,
	type: "string",
	defwidth: 20
	}, { 
	name:"Newsgroups",
	displayName:"Newsgroups",
	re: /^Newsgroups:\s(.*)$/,
	type: "string",
	defwidth: 18
	}, {
	name:"MessageID",
	displayName:"Message-ID",
	re: /^Message-ID:\s(.*)$/,
	type: "string",
	defwidth: 18
	}, {
	name:"Date",
	displayName:"Date",
	re: /^Date:\s(.*)$/,
	type: "datetime",
	defwidth: 10
	}]
}

// -------------------- Event

// Called by Directory Opus to initialize the script.
function OnInit(initData) {
	var uid = "60149ff2-5d64-4d4d-b63b-837d05500e24";
	var url = "https://resource.dopus.com/t/column-eml-msg-email-files-columns/20780";
	initData.url = url;
	initData.name = "Columns for Electronic Mail Format";
	initData.desc = "Adds columns for displaying EML file properties, I.E. To address, from address, subject.";
	initData.copyright = "Initiated by wowbagger; Modified by AndersonNNunes.org";
	initData.version = "1.32";
	initData.default_enable = true;
	initData.min_version = "12.0";
	
	// Helper Function
	// Easy way to determine settings.
	// Thanks goes to "tbone". 
	// https://resource.dopus.com/t/helper-confighelper-easier-config-item-handling/19129
	function ConfigHelper(data) {
		var t=this; t.d=data; t.c=data.config; t.cd=DOpus.Create.Map();
		t.add=function(name, val, des){ t.l={n:name,ln:name.
			toLowerCase()}; return t.val(val).des(des);}
		t.des=function(des){ if (!des) return t; if (t.cd.empty)
			t.d.config_desc=t.cd; t.cd(t.l.n)=des; return t;}
		t.val=function(val){ var l=t.l; if (l.v!==l.x&&typeof l.v=="object")
			l.v.push_back(val);else l.v=t.c[l.n]=val;return t;}
		t.trn=function(){return t.des(t("script.config."+t.l.ln));}
	}
	
	// Helper object.
	var cfg = new ConfigHelper(initData);
	
	// Configuration.
	cfg.add("verbose", true, "Enable output of informative log? If set to false, only errors will print messages.");
	cfg.add("facilitateSubjectGrouping", true, "When enabled, grouping by subject ignores certain prefixes so that all messages from a thread are grouped together.");
	cfg.add("prefixRegExp", "(RES|ENC|Re|Fwd): ", "RegExp used to identify the prefixes to be ignored if the user enabled the 'facilitateSubjectGrouping' option.");
	
	for (i = 0; i < config.columns.length; i++) {
	AddFolderDateCol(initData, config.columns[i].name, config.columns[i].displayName, config.columns[i].type, config.columns[i].defwidth);
	}
}

// Helper Function
// Add column.
function AddFolderDateCol(initData, name, label, type, defwidth) {
	var col;
	col = initData.AddColumn();
	col.name = name;
	col.label = "(Mail) " + label;
	col.header = label;
	col.type = type;
	col.method = "OnEmlEmail";
	if (name == "Subject") {
		col.autogroup = false;
	} else {
		col.autogroup = true;
	}
	col.autorefresh = true;
	col.justify = "left";
	col.multicol = true;
	col.keyscroll = true;
	if (defwidth != null && defwidth != 0) {
		col.defwidth=defwidth;
	}
}

// Helper Function
// Generate value for a column.
function OnEmlEmail(scriptColData) {
	var itemPath = scriptColData.item.realpath;
	if (String(itemPath).length > 259) {
		error("Path length is too long for the file \"" + itemPath + "\".");
		return;
	}
	if (!InArray(scriptColData.col,config.columns)) return;
	if (scriptColData.item.is_dir) return;
	if (scriptColData.item.ext.toLowerCase().match(/\.(eml|msg|txt)/))
	{
		var item = ScanEmailFile(itemPath);
		if (item) {
			for (i = 0; i < config.columns.length; i++) {
				
				// Handle "datetime" column.
				if (config.columns[i].type == "datetime") {
					scriptColData.columns(config.columns[i].name).value = new Date(Date.parse(item[config.columns[i].name])).getVarDate();
				} // Handle other columns.
				else {
					// Value to be assigned to the column.
					var iValue = item[config.columns[i].name];
					
					// For the "Subject" column, remove common prefixes of the group value, if the user desires that.
					if (config.columns[i].name == "Subject") {
						if (Script.config.facilitateSubjectGrouping) {
							scriptColData.columns(config.columns[i].name).group = iValue.replace(new RegExp(Script.config.prefixRegExp, "gi"), "")
						}
					}
					scriptColData.columns(config.columns[i].name).value = iValue;
				}
			}
		}
	}
}

// Helper Function
// Extract value from mail file.
function ScanEmailFile(path, filterFunc) {
	var result = {};

	for (i = 0; i < config.columns.length; i++)
	{
		// Open file for reading only.
		var fn = fs.OpenTextFile(path, 1);
		while (!fn.AtEndOfStream)
		{
			var textLine = fn.ReadLine();
			var match = config.columns[i].re.exec(textLine);
			if (match != null)
			{
				// Try to decode the text if it is encoded.
				
				// This temporary variable is used by decoding function to hold a value that may be necessary to decode text broken over multiple lines.
				var encoding;
				
				// Decode the first line.
				match[1] = decodedLine(match[1], encoding);
				
				// Try to detect if the text is broken over multiple lines and decode them.
				var subsequentLine = fn.ReadLine();
				while (subsequentLine.match(/^(\s|\t).*$/)) {
					match[1] += decodedLine(subsequentLine.slice(1), encoding);
					subsequentLine = fn.ReadLine();
				}
				
				result[config.columns[i].name] = match[1];
				fn.Close();
				break;
			}
		result[config.columns[i].name] = "";
		}
		fn.Close();
	}

	if (result)
		return result;
	else
		return null;
}

// Helper Function
// Check that array contains a given value.
function InArray(col, arr) {
	for (var i = 0; i < arr.length; i++) {
		if (col == arr[i].name) return true;
	}
}

// Helper Function
// Decode line of text.
// The functionality involving encodingPrefix is not implemented yet. That prevents the script from handling one corner case of text broken over multiple lines.
function decodedLine(text, encodingPrefix) {
	var ENCODED_PART_REGEX_PATTERN = /(=\?([^?]+)\?([^?]+)\?)([^?]+)\?=/;
	var matchResult = ENCODED_PART_REGEX_PATTERN.exec(text);
	if (matchResult != null) {
		var front = middle = back = "";
		var firstIndex = matchResult.index;
		var lastIndex = ENCODED_PART_REGEX_PATTERN.lastIndex;
		
		if (firstIndex !== 0) {
			front = text.slice(0, firstIndex);
		}
		
		try {
			middle = st.Decode(matchResult[0].slice(0, -2), "auto");
		}
		catch (err) {
			if (Script.config.verbose) {
				return "[Unsupported Encoding]";
			} else {
				return "";
			}
		}
		
		
		if (lastIndex !== text.length - 1) {
			back = text.slice(lastIndex);
		}
		
		return String(front + middle + back);
	} 
	
	return text;
}

// Helper Function
// Print only if requested.
function print(text) {
	if (Script.config.verbose) {
		DOpus.Output(text);
	}
}

// Helper Function
// Display error message.
function error(text) {
	DOpus.Output(text, true);
}